﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Text
Imports System.Windows.Media

Public Class Sine440AudioStreamSource
    Inherits MediaStreamSource
    ' Standard constants
    Private Const ChannelCount As Integer = 1
    Private Const BitsPerSample As Integer = 16
    Private Const BufferSamples As Integer = 4096 ' can be changed
    Private Const BufferSize As Integer = ChannelCount * BufferSamples * BitsPerSample \ 8

    Private mediaSampleAttributes As Dictionary(Of MediaSampleAttributeKeys, String)
    Private angle As Double
    Private angleIncrement As Double
    Private sampleRate As Integer
    Private timestamp As Long
    Private mediaStreamDescription As MediaStreamDescription
    Private memoryStream As MemoryStream

    Public Sub New(ByVal sampleRate As Integer)
        Me.sampleRate = sampleRate

        ' Want to play A = 440Hz
        angleIncrement = 2 * Math.PI * 440 / sampleRate

        ' Create empty mediaSampleAttributes dictionary for OpenReadAsync
        mediaSampleAttributes = New Dictionary(Of MediaSampleAttributeKeys, String)

        ' Create re-usable MemoryStream for accumulating audio samples
        memoryStream = New MemoryStream
    End Sub

    Protected Overrides Sub OpenMediaAsync()
        Dim byteRate As Integer = sampleRate * ChannelCount * BitsPerSample \ 8
        Dim blockAlign As Short = CShort(ChannelCount) * CShort(BitsPerSample \ 8)

        ' Build string-based wave-format structure
        Dim waveFormat As String = ""
        waveFormat &= ToLittleEndianString(String.Format("{0:X4}", 1)) ' indicates PCM
        waveFormat &= ToLittleEndianString(String.Format("{0:X4}", ChannelCount))
        waveFormat &= ToLittleEndianString(String.Format("{0:X8}", sampleRate))
        waveFormat &= ToLittleEndianString(String.Format("{0:X8}", byteRate))
        waveFormat &= ToLittleEndianString(String.Format("{0:X4}", blockAlign))
        waveFormat &= ToLittleEndianString(String.Format("{0:X4}", BitsPerSample))
        waveFormat &= ToLittleEndianString(String.Format("{0:X4}", 0))

        ' Put wave format string in media streams dictionary
        Dim mediaStreamAttributes = New Dictionary(Of MediaStreamAttributeKeys, String)
        mediaStreamAttributes(MediaStreamAttributeKeys.CodecPrivateData) = waveFormat

        ' Make description to add to available streams list
        Dim availableMediaStreams = New List(Of MediaStreamDescription)
        mediaStreamDescription = New MediaStreamDescription(MediaStreamType.Audio, mediaStreamAttributes)
        availableMediaStreams.Add(mediaStreamDescription)

        ' Set some appropriate keys in the media source dictionary
        Dim mediaSourceAttributes = New Dictionary(Of MediaSourceAttributesKeys, String)
        mediaSourceAttributes(MediaSourceAttributesKeys.Duration) = "0"
        mediaSourceAttributes(MediaSourceAttributesKeys.CanSeek) = "false"

        ' Signal that the open operation is completed
        ReportOpenMediaCompleted(mediaSourceAttributes, availableMediaStreams)
    End Sub

    ' For building string-based wave-format structure
    Private Function ToLittleEndianString(ByVal bigEndianString As String) As String
        Dim strBuilder As New StringBuilder

        For i = 0 To bigEndianString.Length - 1 Step 2
            strBuilder.Insert(0, bigEndianString.Substring(i, 2))
        Next i

        Return strBuilder.ToString
    End Function
    ' Provides audio samples from AudioSampleProvider property.
    '  (MediaStreamType parameter will always equal Audio.)
    Protected Overrides Sub GetSampleAsync(ByVal mediaStreamType As MediaStreamType)
        ' Reset MemoryStream object
        memoryStream.Seek(0, SeekOrigin.Begin)

        For sample = 0 To BufferSamples - 1
            Dim amplitude As Short = CShort(Short.MaxValue) * CShort(Math.Sin(angle))
            memoryStream.WriteByte(CByte(amplitude And &HFF))
            Dim temp = amplitude >> 8
            If temp < 0 Then
                memoryStream.WriteByte(Math.Abs(( temp Xor &HFF) + 1))
            Else
                memoryStream.WriteByte(CByte(temp))
            End If

            angle = (angle + angleIncrement) Mod (2 * Math.PI)
        Next sample

        ' Send out the sample
        ReportGetSampleCompleted(New MediaStreamSample(mediaStreamDescription,
                                                       memoryStream,
                                                       0,
                                                       BufferSize,
                                                       timestamp,
                                                       mediaSampleAttributes))
        ' Prepare for next sample
        timestamp += BufferSamples * 10000000L / sampleRate
    End Sub

    Protected Overrides Sub SeekAsync(ByVal seekToTime As Long)
        ReportSeekCompleted(seekToTime)
    End Sub

    Protected Overrides Sub CloseMedia()
        mediaStreamDescription = Nothing
    End Sub

    ' Shouldn't get a call here because only one MediaStreamDescription is supported
    Protected Overrides Sub SwitchMediaStreamAsync(ByVal mediaStreamDescription As MediaStreamDescription)
        Throw New NotImplementedException
    End Sub

    Protected Overrides Sub GetDiagnosticAsync(ByVal diagnosticKind As MediaStreamSourceDiagnosticKind)
        Throw New NotImplementedException
    End Sub
End Class
