﻿Imports Microsoft.Phone.BackgroundAudio
Imports Petzold.MusicSynthesis


''' <summary>
''' A background agent that performs per-track streaming for playback
''' </summary>
Public Class AudioTrackStreamer
    Inherits AudioStreamingAgent
    Private Const SAMPLE_RATE As Integer = 44100

    ''' <summary>
    ''' Called when a new track requires audio decoding
    ''' (typically because it is about to start playing)
    ''' </summary>
    ''' <param name="track">
    ''' The track that needs audio streaming
    ''' </param>
    ''' <param name="streamer">
    ''' The AudioStreamer object to which a MediaStreamSource should be
    ''' attached to commence playback
    ''' </param>
    ''' <remarks>
    ''' To invoke this method for a track set the Source parameter of the AudioTrack to null
    ''' before setting  into the Track property of the BackgroundAudioPlayer instance
    ''' property set to true;
    ''' otherwise it is assumed that the system will perform all streaming
    ''' and decoding
    ''' </remarks>
    Protected Overrides Sub OnBeginStreaming(ByVal track As AudioTrack, ByVal streamer As AudioStreamer)
        ' Create a Rambler
        ' Maximum -  Minimum -  Start
        Dim rambler As New Rambler(SAMPLE_RATE, New Pitch(Note.Csharp, 4), _
                                   New Pitch(Note.Csharp, 2), _
                                   New Pitch(Note.Csharp, 6)) _
                               With {.Oscillator = New AlmostSquareWave(SAMPLE_RATE), .Tempo = 480}


        ' Set allowable note values
        rambler.Notes.Add(Note.Csharp)
        rambler.Notes.Add(Note.Dsharp)
        rambler.Notes.Add(Note.Fsharp)
        rambler.Notes.Add(Note.Gsharp)
        rambler.Notes.Add(Note.Asharp)


        ' Create Mixer and MixerInput objects
        Dim mixer As New Mixer
        mixer.Inputs.Add(New MixerInput(rambler))

        Dim audioStreamSource As New DynamicPcmStreamSource(SAMPLE_RATE)
        audioStreamSource.SampleProvider = mixer
        streamer.SetSource(audioStreamSource)
    End Sub

    ''' <summary>
    ''' Called when the agent request is getting cancelled
    ''' The call to base.OnCancel() is necessary to release the background streaming resources
    ''' </summary>
    Protected Overrides Sub OnCancel()
        MyBase.OnCancel()
    End Sub
End Class
