﻿Imports System
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media


Partial Public Class Waveformer
    Inherits UserControl
    Private Const NUM_POINTS As Integer = 15
    Private touchDictionary As New Dictionary(Of Integer, WaveformerNode)

    Public Sub New()
        InitializeComponent()

        Me.Points = New FixedLengthCollection(Of Point)(NUM_POINTS)
        AddHandler Points.CollectionChanged, AddressOf OnPointsCollectionChanged

        For i As Integer = 0 To NUM_POINTS - 1
            canvas.Children.Add(New WaveformerNode)
            polyline.Points.Add(New Point)
        Next i

        ' Add two more to Polyline for beginning and end
        polyline.Points.Add(New Point)
        polyline.Points.Add(New Point)

        AddHandler SizeChanged, AddressOf OnSizeChanged
        AddHandler Touch.FrameReported, AddressOf OnTouchFrameReported
    End Sub

    Private privatePoints As FixedLengthCollection(Of Point)
    Public Property Points As FixedLengthCollection(Of Point)
        Get
            Return privatePoints
        End Get
        Protected Set(ByVal value As FixedLengthCollection(Of Point))
            privatePoints = value
        End Set
    End Property

    Private Sub OnSizeChanged(ByVal sender As Object, ByVal args As SizeChangedEventArgs)
        Synchronize()
    End Sub

    Private Sub OnPointsCollectionChanged(ByVal sender As Object, ByVal args As NotifyCollectionChangedEventArgs)
        Synchronize()
    End Sub

    Private Sub Synchronize()
        ' First and last points in Polyline are on axis
        polyline.Points(0) = RelativeToAbsolute(New Point(0, 0))
        polyline.Points(NUM_POINTS + 1) = RelativeToAbsolute(New Point(2 * Math.PI, 0))

        For index As Integer = 0 To NUM_POINTS - 1
            Dim point As Point = RelativeToAbsolute(Me.Points(index))
            TryCast(canvas.Children(index), WaveformerNode).Point = point
            polyline.Points(index + 1) = point
        Next index
    End Sub

    Private Sub OnTouchFrameReported(ByVal sender As Object, ByVal args As TouchFrameEventArgs)
        Dim primaryTouchPoint As TouchPoint = args.GetPrimaryTouchPoint(waveLayoutPanel)

        ' Don't let usable touch points get promoted to mouse events
        If primaryTouchPoint IsNot Nothing AndAlso primaryTouchPoint.Action = TouchAction.Down AndAlso TypeOf GetParent(Of WaveformerNode)(primaryTouchPoint.TouchDevice.DirectlyOver) Is WaveformerNode Then
            args.SuspendMousePromotionUntilTouchUp()
        End If

        ' Enumerate all the TouchPoints
        Dim touchPoints As TouchPointCollection = args.GetTouchPoints(waveLayoutPanel)

        For Each touchPoint As TouchPoint In touchPoints
            Dim touchDevice As TouchDevice = touchPoint.TouchDevice
            Dim id As Integer = touchDevice.Id
            Dim position As Point = touchPoint.Position

            Select Case touchPoint.Action
                Case TouchAction.Down
                    Dim node As WaveformerNode = GetParent(Of WaveformerNode)(touchDevice.DirectlyOver)

                    If node IsNot Nothing Then
                        ' Shouldn't happen but let it go
                        If touchDictionary.ContainsKey(id) Then
                            touchDictionary(id) = node
                        Else
                            ' Normal case
                            touchDictionary.Add(id, node)
                        End If

                        SetPointForNode(node, position)
                    End If

                Case TouchAction.Move
                    If touchDictionary.ContainsKey(id) Then
                        SetPointForNode(touchDictionary(id), position)
                    End If

                Case TouchAction.Up
                    If touchDictionary.ContainsKey(id) Then
                        touchDictionary.Remove(id)
                    End If
            End Select
        Next touchPoint
    End Sub

    Private Function GetParent(Of T As DependencyObject)(ByVal element As DependencyObject) As T
        Dim parent As DependencyObject = VisualTreeHelper.GetParent(element)

        If parent Is Nothing Then
            Return Nothing
        End If

        If TypeOf parent Is T Then
            Return TryCast(parent, T)
        End If

        Return GetParent(Of T)(parent)
    End Function

    Private Sub SetPointForNode(ByVal node As WaveformerNode, ByVal position As Point)
        Dim childIndex As Integer = canvas.Children.IndexOf(node)
        Dim x As Double = Me.Points(childIndex).X
        Dim y As Double = AbsoluteToRelativeY(position.Y)
        Me.Points(childIndex) = New Point(x, y)
    End Sub

    Private Function RelativeToAbsolute(ByVal point As Point) As Point
        Return New Point(waveLayoutPanel.ActualWidth * point.X / (2 * Math.PI), waveLayoutPanel.ActualHeight * (1 - point.Y) / 2)
    End Function

    Private Function AbsoluteToRelativeY(ByVal y As Double) As Double
        Return Math.Max(-1, Math.Min(1, 1 - 2 * y / waveLayoutPanel.ActualHeight))
    End Function
End Class
