﻿Public Structure Pitch
    Private Shared ReadOnly semitone As Double = Math.Pow(2, 1.0 / 12)

    Private Shared ReadOnly middleC As Double = 440 * Math.Pow(semitone, 3) / 2

    ' Middle C is 64
    Public Sub New(ByVal midiNumber As Integer)
        Me.New(CType(midiNumber Mod 12, Note), midiNumber \ 12 - 1)
    End Sub

    Public Sub New(ByVal note As Note, ByVal octave As Integer)
        Me.Note = note
        Me.Octave = octave
        Me.MidiNumber = 12 * (Me.Octave + 1) + CInt(Fix(Me.Note))
        Me.Frequency = middleC * Math.Pow(semitone, CInt(Fix(Me.Note))) * Math.Pow(2, Me.Octave - 4)
    End Sub

    Private _note As Note
    Public Property Note As Note
        Get
            Return _note
        End Get
        Private Set(ByVal value As Note)
            _note = value
        End Set
    End Property
    Private _octave As Integer
    Public Property Octave As Integer
        Get
            Return _octave
        End Get
        Private Set(ByVal value As Integer)
            _octave = value
        End Set
    End Property
    Private _midiNumber As Integer
    Public Property MidiNumber As Integer
        Get
            Return _midiNumber
        End Get
        Private Set(ByVal value As Integer)
            _midiNumber = value
        End Set
    End Property
    Private _frequency As Double
    Public Property Frequency As Double
        Get
            Return _frequency
        End Get
        Private Set(ByVal value As Double)
            _frequency = value
        End Set
    End Property

    Public Overrides Function ToString() As String
        Dim note As String = Me.Note.ToString

        Return String.Format("{0}{1}{2} {3} ({4}) {5}Hz", note.Chars(0), If(note.EndsWith("sharp"), ChrW(&H266F).ToString, ""), If(note.EndsWith("flat"), ChrW(&H266D).ToString, ""), Me.Octave, Me.MidiNumber, Me.Frequency)
    End Function

End Structure
