﻿Imports System
Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Windows
Imports System.Windows.Media
Imports System.Windows.Navigation
Imports System.Xml.Serialization
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Petzold.MusicSynthesis


Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private Const SAMPLE_RATE As Integer = 22050
    Private mixer As Mixer

    Public Sub New()
        InitializeComponent()

        ' Get app bar buttons
        playButton = TryCast(Me.ApplicationBar.Buttons(0), ApplicationBarIconButton)
        pauseButton = TryCast(Me.ApplicationBar.Buttons(1), ApplicationBarIconButton)

        ' Initialize Waveformer control
        For i = 0 To waveformer.Points.Count - 1
            Dim angle As Double = (i + 1) * 2 * Math.PI / (waveformer.Points.Count + 1)
            waveformer.Points(i) = New Point(angle, Math.Sin(angle))
        Next i

        ' Create two Sequencers with slightly different tempi
        Dim sequencer1 As New Sequencer(SAMPLE_RATE) With {.Oscillator = New VariableWaveformOscillator(SAMPLE_RATE) With {.Points = waveformer.Points}, .Tempo = 480}

        Dim sequencer2 As New Sequencer(SAMPLE_RATE) With {.Oscillator = New VariableWaveformOscillator(SAMPLE_RATE) With {.Points = waveformer.Points}, .Tempo = 470}

        ' Set the same Pitches in the Sequencers
        Dim pitches() As Pitch = {New Pitch(Note.E, 4), New Pitch(Note.G, 4), New Pitch(Note.B, 4), New Pitch(Note.E, 5), New Pitch(Note.B, 4), New Pitch(Note.G, 4), New Pitch(Note.A, 4), New Pitch(Note.C, 5), New Pitch(Note.E, 5), New Pitch(Note.A, 5), New Pitch(Note.E, 5), New Pitch(Note.C, 5), New Pitch(Note.D, 4), New Pitch(Note.F, 4), New Pitch(Note.A, 4), New Pitch(Note.D, 5), New Pitch(Note.A, 4), New Pitch(Note.F, 4), New Pitch(Note.G, 4), New Pitch(Note.B, 4), New Pitch(Note.D, 5), New Pitch(Note.G, 5), New Pitch(Note.D, 5), New Pitch(Note.B, 4)}

        For Each pitch As Pitch In pitches
            sequencer1.Pitches.Add(pitch)
            sequencer2.Pitches.Add(pitch)
        Next pitch

        ' Create Mixer and MixerInput objects
        mixer = New Mixer()
        mixer.Inputs.Add(New MixerInput(sequencer1) With {.Space = -0.5})
        mixer.Inputs.Add(New MixerInput(sequencer2) With {.Space = 0.5})
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
        Dim points(waveformer.Points.Count - 1) As Point

        Using storage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
            If storage.FileExists("settings.xml") Then
                ' Load Waveformer Points
                Dim xmlSerializer As New XmlSerializer(GetType(Point))
                Using stream As IsolatedStorageFileStream = storage.OpenFile("settings.xml", FileMode.Open)
                    points = TryCast(xmlSerializer.Deserialize(stream), Point())
                End Using
            Else
                ' Or initialize them to sine curve
                For i = 0 To waveformer.Points.Count - 1
                    Dim angle As Double = (i + 1) * 2 * Math.PI / (waveformer.Points.Count + 1)
                    points(i) = New Point(angle, Math.Sin(angle))
                Next i
            End If
        End Using

        For i = 0 To waveformer.Points.Count - 1
            waveformer.Points(i) = points(i)
        Next i

        ' Create MediaStreamSource derivative & set in MediaElement
        Dim dynamicPcmStreamSource As New DynamicPcmStreamSource(SAMPLE_RATE)
        dynamicPcmStreamSource.SampleProvider = mixer
        MediaElement.SetSource(dynamicPcmStreamSource)

        MyBase.OnNavigatedTo(e)
    End Sub

    Protected Overrides Sub OnNavigatedFrom(ByVal args As NavigationEventArgs)
        ' Save Waveformer Points
        Dim points(waveformer.Points.Count - 1) As Point
        waveformer.Points.CopyTo(points, 0)
        Dim xmlSerializer As New XmlSerializer(GetType(Point()))

        Using storage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
            Using stream As IsolatedStorageFileStream = storage.CreateFile("settings.xml")
                xmlSerializer.Serialize(stream, points)
            End Using
        End Using

        MyBase.OnNavigatedFrom(args)
    End Sub

    Private Sub OnMediaElementCurrentStateChanged(ByVal sender As Object, ByVal args As RoutedEventArgs)
        playButton.IsEnabled = MediaElement.CurrentState <> MediaElementState.Playing
        pauseButton.IsEnabled = MediaElement.CurrentState = MediaElementState.Playing
    End Sub

    Private Sub OnPlayButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        MediaElement.Play()
        playButton.IsEnabled = False
    End Sub

    Private Sub OnPauseButtonClick(ByVal sender As Object, ByVal args As EventArgs)
        MediaElement.Pause()
        pauseButton.IsEnabled = False
    End Sub
End Class
